@@ -11,6 +11,7 @@ import traceback
 
 config_file_exist = config.exist()
 current_path = os.path.dirname(os.path.abspath(__file__))
+prefix = '.env/bin/' if util.is_venv() or config.has_venv() else ''
 
 
 def ignore(fn: Callable, value):
@@ -58,13 +59,13 @@ def init(args):
     '''
     python = args.python or 'python3'
     config.write(dict(python=python))
-    config.write(dict(python_exec='.env/bin/python'))
+    config.write(dict(python_exec=prefix + 'python'))
     os.system('virtualenv .env --python=%s' % python)
-    os.system('.env/bin/pip install sl_pip')
-    os.system('.env/bin/pip install ipython coverage flake8 nose coverage')
+    os.system(prefix + 'pip install sl_pip')
+    os.system(prefix + 'pip install ipython coverage flake8 nose coverage')
     if python == 'python3':
         util.info('Installing stack inside venv')
-        os.system('.env/bin/pip install python_stack')
+        os.system(prefix + 'pip install python_stack --upgrade')
 
 
 @as_command
@@ -72,7 +73,7 @@ def setup(args):
     '''
     Install libs from requirements.txt to venv
     '''
-    ignore(os.system, '.env/bin/pip install -r ./requirements.txt --process-dependency-links')
+    ignore(os.system, prefix + 'pip install -r ./requirements.txt --process-dependency-links')
     projectname = os.path.split(os.path.dirname(os.path.realpath(__name__)))[-1]
     config.write(dict(project=projectname))
 
@@ -86,15 +87,15 @@ def install(args):
     @argument --git, metavar=git, help=Decare is a git repo, default=
     '''
     if not args.lib:
-        ignore(os.system, '.env/bin/pip install -r ./requirements.txt --process-dependency-links')
+        ignore(os.system, prefix + 'pip install -r ./requirements.txt --process-dependency-links')
     git = bool(args.repo)
     if not git:
-        os.system('.env/bin/pip install %s -v --process-dependency-links' % args.lib)
+        os.system(prefix + 'pip install %s -v --process-dependency-links' % args.lib)
     if git:
         template = config.load().get('git_path', 'git+{repo}#egg={lib}')
         repo = template.format(**dict(repo=args.repo, lib=args.lib))
-        os.system('.env/bin/pip install -e %s --process-dependency-links' % repo)
-    os.system('.env/bin/pip freeze > requirements.txt')
+        os.system(prefix + 'pip install -e %s --process-dependency-links' % repo)
+    os.system(prefix + 'pip freeze > requirements.txt')
 
 
 @as_command
@@ -103,7 +104,7 @@ def uninstall(args):
     Uninstall libs
     @argument lib, metavar=LIB, help=Lib name
     '''
-    return os.system('.env/bin/pip uninstall %s' % args.lib)
+    return os.system(prefix + 'pip uninstall %s' % args.lib)
 
 
 @as_command
@@ -111,17 +112,19 @@ def installed(args):
     '''
     List installed libs
     '''
-    import pip
-    list(map(print, pip.commands.freeze.freeze()))
+    if util.is_venv():
+        import pip
+        list(map(print, pip.commands.freeze.freeze()))
+    else:
+        os.system(prefix + 'pip freeze')
 
 
 @as_command
-def fabric(args):
+def fab(args):
     '''
     drop to Fabric
     '''
-    import fabric.main
-    return fabric.main.main([os.path.abspath(__file__)])
+    os.system(prefix + 'fab %s' % ' '.join(sys.argv[2:]))
 
 
 @as_command
@@ -129,7 +132,7 @@ def test(args):
     '''
     run unittest
     '''
-    return os.system('.env/bin/nosetests -sv')
+    return os.system(prefix + 'nosetests -sv')
 
 
 @as_command
@@ -138,7 +141,15 @@ def coverage(args):
     Report unittest coverage
     '''
     project = config.load().get('project')
-    return os.system('.env/bin/nosetests -sv --with-coverage --cover-package %s' % project)
+    return os.system(prefix + 'nosetests -sv --with-coverage --cover-package %s' % project)
+
+
+@as_command
+def run(args):
+    '''
+    exec file
+    '''
+    return os.system(prefix + 'python %s' % ' '.join(sys.argv[2:]))
 
 
 @as_command
@@ -146,7 +157,7 @@ def python(args):
     '''
     run python
     '''
-    return os.system('.env/bin/python %s' % ' '.join(sys.argv[2:]))
+    return os.system(prefix + 'python %s' % ' '.join(sys.argv[2:]))
 
 
 @as_command
@@ -154,7 +165,7 @@ def repl(args):
     '''
     call ipython as repl
     '''
-    return os.system('.env/bin/ipython')
+    return os.system(prefix + 'ipython')
 
 
 @as_command
@@ -162,7 +173,7 @@ def pip(args):
     '''
     call pip
     '''
-    return os.system('.env/bin/pip %s' % ' '.join(sys.argv[2:]))
+    return os.system(prefix + 'pip %s' % ' '.join(sys.argv[2:]))
 
 
 @as_command
@@ -193,6 +204,7 @@ def router(argv) -> Callable:
         'setup': setup,
         'python': python,
         'init': init,
+        'run': run,
         'clear': clear,
         'installed': installed,
         'uninstall': uninstall,
@@ -200,22 +212,19 @@ def router(argv) -> Callable:
         'pass': uninstall,
         'serve': git_serve,
         'coverage': coverage,
-        'doc': doc
-    }.get(argv[1], fabric)(args)
+        'doc': doc,
+        'fab': fab
+    }.get(argv[1], fab)(args)
 
 
 def main():
-    if len(sys.argv) > 1 and not util.is_venv() and not sys.argv[1] == 'init':
+    if len(sys.argv) > 1 and not config.has_venv() and not sys.argv[1] == 'init':
         util.warn('Warning: Command running outside the venv, you may need to run `stack init` first')
         util.warn('Warning: Using lib path %s' % sysconfig.get_path('platlib'))
-        if config.has_venv() and config.load().get('python', '') == 'python3':
-            util.info('Try switching to .env/bin/python')
-            os.system('.env/bin/stack %s' % ''.join(sys.argv[1:]))
-    else:
-        try:
-            router(sys.argv)
-        except Exception as ex:
-            util.error("Exception <%s>, Traceback: %r", str(ex), traceback.format_exc())
+    try:
+        router(sys.argv)
+    except Exception as ex:
+        util.error("Exception <%s>, Traceback: %r" % (str(ex), traceback.format_exc()))
 
 if __name__ == '__main__':
     main()
